#!/usr/bin/env node
var __commonJS = (callback, module2) => () => {
  if (!module2) {
    module2 = {exports: {}};
    callback(module2.exports, module2);
  }
  return module2.exports;
};

// src/build/static.js
var require_static = __commonJS((exports2, module2) => {
  var {
    resolve: resolve2,
    posix: {join}
  } = require("path");
  var {copy, readFile, writeFile, exists} = require("fs-extra");
  var postcss = require("postcss");
  var fg = require("fast-glob");
  var buildStatic = async (_, outdir) => {
    const basedir = resolve2(outdir);
    if (await exists(resolve2("public")))
      await copy(resolve2("public"), basedir);
    const cssFiles = await fg(join(outdir, "/**/*.css"));
    let postcssPlugins = null;
    if (await exists(resolve2("postcss.config.js"))) {
      const postcssConfig = require(resolve2("postcss.config.js"));
      postcssPlugins = postcssConfig.plugins;
    }
    if (postcssPlugins && postcssPlugins !== null) {
      const cssProcessor = postcss(postcssPlugins);
      for (let file of cssFiles) {
        const filePath = resolve2(file);
        const result = await cssProcessor.process(await readFile(filePath), {
          from: filePath
        });
        await writeFile(filePath, result.css);
      }
    }
  };
  module2.exports = buildStatic;
});

// src/build/document.js
var require_document = __commonJS((exports2, module2) => {
  var {
    join,
    resolve: resolve2,
    posix: {join: posixJoin}
  } = require("path");
  var fg = require("fast-glob");
  var LoadCustomDocument = (document2, indir, outdir) => {
    const basedir = resolve2(outdir);
    if (fg.sync(posixJoin(outdir, indir, "_document.js")).length) {
      const customDocument = require(join(basedir, indir, "_document.js")).default();
      const bodyEl = customDocument.getElementsByTagName("body")[0];
      const headEl = customDocument.getElementsByTagName("head")[0];
      Object.entries(customDocument._attributes[null]).forEach(([key, value]) => {
        document2.getElementsByTagName("html")[0].setAttribute(key, value.value.toString());
      });
      Array.from(bodyEl.childNodes).forEach((childNode) => {
        document2.body.appendChild(childNode);
      });
      Array.from(headEl.childNodes).forEach((childNode) => {
        document2.head.appendChild(childNode);
      });
    } else {
      const containerDiv = document2.createElement("div");
      containerDiv.className += "root";
      document2.body.appendChild(containerDiv);
    }
  };
  var initDocument = (indir, outdir) => {
    const document2 = require("min-document");
    document2.body.childNodes = [];
    document2.body.innerHTML = "";
    document2.head.childNodes = [];
    document2.head.innerHTML = "";
    LoadCustomDocument(document2, indir, outdir);
    return document2;
  };
  module2.exports = initDocument;
});

// src/build/js.js
var require_js = __commonJS((exports2, module2) => {
  var {
    join,
    resolve: resolve2,
    dirname,
    posix: {join: posixJoin, normalize: posixNormalize}
  } = require("path");
  var {ensureFile, writeFile, remove} = require("fs-extra");
  var {startService} = require("esbuild");
  var fg = require("fast-glob");
  var newDocument = require_document();
  var buildJS = async (indir, outdir) => {
    const basedir = resolve2(outdir);
    const service = await startService();
    global.headContents = [];
    try {
      let writePageDOM = async (pageDOM, path) => {
        document = newDocument(indir, outdir);
        const rootEl = document.getElementsByClassName("root")[0] || document.body;
        if (Array.isArray(pageDOM)) {
          pageDOM.forEach((singleEl) => {
            rootEl.appendChild(singleEl);
          });
        } else {
          rootEl.appendChild(pageDOM);
        }
        for (var i = global.headContents.length - 1; i >= 0; i--) {
          document.head.insertBefore(global.headContents[i], document.head.childNodes[0]);
        }
        await ensureFile(path);
        await writeFile(path, `<!DOCTYPE html>` + document.documentElement.toString());
        rootEl.removeChild(pageDOM);
        for (let node of global.headContents) {
          document.head.removeChild(node);
        }
        global.headContents = [];
      };
      const jsFiles = await fg(posixJoin(indir, "/**/*.js"));
      const services = jsFiles.map(async (file) => service.build({
        entryPoints: [file],
        outfile: join(basedir, file),
        bundle: true,
        platform: "node",
        format: "cjs",
        loader: {
          ".js": "jsx"
        },
        jsxFactory: "Explosiv.el",
        jsxFragment: "Explosiv.fragment",
        inject: [resolve2(__dirname, "./explosiv.shim.js")]
      }));
      await Promise.all(services);
      let pages = await fg(posixJoin(outdir, indir, "/**/*.js"));
      pages = pages.filter((page) => page !== posixJoin(outdir, indir, "_document.js"));
      for (let page of pages) {
        const fileExports = require(resolve2(page));
        const filePath = posixNormalize(page).split("/").slice(1 + posixNormalize(indir).split("/").length).join("/").slice(0, -3);
        let writeComponent = async (path = null) => {
          const props = fileExports.getProps ? await fileExports.getProps(path) : {};
          let src;
          if (path)
            src = join(dirname(filePath), path);
          else
            src = filePath;
          if (src.endsWith("/index"))
            src = src + ".html";
          src = join(basedir, src, src.endsWith("html") ? "" : "index.html");
          await writePageDOM(fileExports.default(props), src);
        };
        if (typeof fileExports.default !== "function")
          throw `Default export from a file in ${indir} must be a funtion`;
        if (typeof fileExports.getPaths === "function") {
          const paths = await fileExports.getPaths();
          for (let path of paths) {
            await writeComponent(path);
          }
        } else {
          await writeComponent();
        }
      }
    } catch (err) {
      throw err;
    } finally {
      await remove(join(basedir, indir));
      service.stop();
    }
  };
  module2.exports = buildJS;
});

// src/build/builds.js
var require_builds = __commonJS((exports2, module2) => {
  module2.exports = {
    static: require_static(),
    js: require_js()
  };
});

// src/build.js
var require_build = __commonJS((exports2, module2) => {
  var builds = require_builds();
  async function build2(indir, outdir) {
    global.headContents = [];
    require("dotenv").config();
    for (const file of Object.keys(require.cache)) {
      delete require.cache[file];
    }
    for (thread in builds) {
      await builds[thread](indir, outdir);
    }
  }
  module2.exports = build2;
});

// src/cli.js
var {resolve} = require("path");
var chokidar = require("chokidar");
var ora = require("ora");
var sirv = require("sirv");
var compression = require("compression");
var connect = require("connect");
var morgan = require("morgan");
var http = require("http");
var {readFileSync} = require("fs");
var chalk = require("chalk");
var sade = require("sade");
var build = require_build();
var {removeSync} = require("fs-extra");
var cli = sade("explosiv");
cli.version(JSON.parse(readFileSync(resolve(__dirname, "..", "package.json"))).version);
cli.command("dev").describe("Start the dev server, & rebuilds static files on file change").action(explosivDev).option("-i, --indir", "Change input directory for your files. (defaults to `./pages`)").option("-d, --devdir", "Change directory where your temporary development builds are stored. (defaults to `./__explosiv__`)").option("-p, --port", "Change port for `dev server`. (defaults to process.env.PORT or `3000`)");
cli.command("serve").describe("Start a static file server").action(explosivServe).option("-d, --dir", "Change directory whom files will be served. (defaults to `./out`)").option("-p, --port", "Change port for `server`. (defaults to process.env.PORT or `3000`)");
cli.command("build").describe("Build production ready static files").action(explosivProd).option("-i, --indir", "Change input directory for your files. (defaults to `./pages`)").option("-o, --outdir", "Change output directory of your `build` command. (defaults to `./out`)");
cli.parse(process.argv);
function explosivServe({dir = "out", port = 3e3}, dev = false) {
  const assets = sirv(resolve(dir), {
    dev: true
  });
  let server = connect().use(compression()).use(morgan("dev")).use(assets);
  http.createServer(server).listen(process.env.PORT || port, (err) => {
    if (err)
      throw err;
    console.log(`${dev ? "Development" : "Static"} server is live at: ${chalk.cyan(`http://localhost:${process.env.PORT || port}`)}
`);
  });
}
function explosivDev({
  indir = "pages",
  devdir = "__explosiv__",
  port = 3e3
}) {
  process.env.NODE_ENV = "development";
  const watcher = chokidar.watch(".", {
    awaitWriteFinish: true,
    ignoreInitial: true,
    ignored: (path) => path.startsWith(devdir) || path.startsWith("node_modules")
  });
  const spinner = ora("Building...");
  const buildWithSpinner = async () => {
    spinner.start();
    try {
      await build(indir, devdir);
      spinner.succeed("Built successfully...");
    } catch (err) {
      spinner.fail(err);
      console.error(err);
    }
  };
  watcher.on("change", async () => await buildWithSpinner());
  watcher.on("add", async () => await buildWithSpinner());
  watcher.on("ready", async () => {
    await buildWithSpinner();
    explosivServe({dir: devdir, port}, true);
  });
  function exitHandler() {
    watcher.close();
    process.exit(0);
  }
  process.on("exit", exitHandler);
  process.on("SIGINT", exitHandler);
  process.on("SIGUSR1", exitHandler);
  process.on("SIGUSR2", exitHandler);
  process.on("uncaughtException", exitHandler);
}
async function explosivProd({indir = "pages", outdir = "out"}) {
  removeSync(outdir);
  process.env.NODE_ENV = "production";
  const spinner = ora("Building production build...").start();
  try {
    await build(indir, outdir);
    spinner.succeed("Built successfully...");
  } catch (err) {
    spinner.fail(err);
    throw err;
  }
}
